/* Cdigo para el clculo de las subbandas de energa en estructuras
   semiconductoras de GaAs con dopado tipo delta.
   Francisco Domnguez-Adame                                          */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

/* Parmetros del sistema. La energa se mide en Rydberg efectivo
   (5.8 meV) y la distancia en radio de Bohr efectivo (100 A)         */

#define NDMIN     0.5   /* Densidad superficial mnima de donadores   */
#define NDMAX     5.0   /* Densidad superficial mxima de donadores   */
#define NDSTEP    0.1   /* Paso en densidad superficial de donadores  */
#define NS        2     /* Nmero de subbandas a determinar           */
#define XMAX     20     /* Distancia mxima de integracin            */
#define N      1000     /* Nmero de intervalos de integracin        */
#define EMAX     -0.05  /* Mximo valor de la energa                 */
#define ESTEP     0.05  /* Paso en energas                           */
#define EPS       1e-4  /* Precisin requerida                        */
#define PI    3.141592654
#define FILENAME  "DATOS.DAT"

double v[N];      /* Almacena los valores del potencial en cada punto */
double liminf[NS];/* Lmite inferior de cada intervalo de biseccin   */
double limsup[NS];/* Lmite superior de cada intervalo de biseccin   */

void potencial(double v[N],double s,double nd) {
   int i;
   double x,alfa=2.0/15.0/PI,x_0,aux;
   x_0=pow(alfa*alfa*alfa/nd/PI,1./5.);
   for(i=1;i<N;i++)  {
      x=-XMAX+s*(double)i;
      aux=alfa*fabs(x)+x_0;
      aux=alfa/aux/aux;
      v[i]=-aux*aux;
   }
}

int nodos(double v[N],double e,double s) {  /* Nmero de ceros        */
   int i,cont=0;
   double p;
   p=2.0+s*s*(v[1]-e);
   for(i=2;i<N;i++) {
      p=2.0+s*s*(v[i]-e)-1.0/p;
      if(p<0) cont++;
   }
   return cont;
}

void main(void) {
   int i,cont,aux;
   double e,s=2.*XMAX/(double)N,e_min,e_max,eps,nd,dnd=NDSTEP;
   FILE *out;
   out=fopen(FILENAME,"w");
   for(nd=NDMIN;nd<=NDMAX;nd+=dnd) {  /* Bucle en densidad de dopado  */
      cont=0;
      potencial(v,s,nd);
      e=v[N/2];
      while(cont<NS) {
         e+=ESTEP;
         if(e>=EMAX+ESTEP) {
            printf("No hay %d subbandas cuando ND=%4.2lf\n",NS,nd);
            exit(1);
         }
         aux=nodos(v,e,s);       /* Calcula el nmero de nodos        */
         if(aux==cont+1) {       /* Determina lmites del intervalo   */
            liminf[cont]=e-ESTEP;/* de bsqueda para el mtodo de la  */
            limsup[cont]=e+ESTEP;/* biseccin para cada nivel         */
            cont=aux;
         }
      }
      fprintf(out,"%3.2lf   ",(double)nd);
      for(i=0;i<NS;i++) {       /* Comienza el mtodo de la biseccin */
         eps=1.e30;             /* para cada nivel                    */
         e_min=liminf[i];
         e_max=limsup[i];
         while(eps>EPS)  {
            e=0.5*(e_max+e_min);
            aux=nodos(v,e,s);
            if(aux==i) e_max=e;
            else e_min=e;
            eps=e_max-e_min;
         }
         e=0.5*(e_max+e_min);
         fprintf(out,"%10.7lf   ",5.8*e);
      }
      fprintf(out,"\n");
   }
   fclose(out);
}
